// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// </summary>
	[Description("OsMA")]
	public class OsMA : Indicator
	{
		#region Variables
		private int					fast	= 12;
		private int					slow	= 26;
		private int					smooth	= 9;
		private	DataSeries		fastEma;
		private	DataSeries		slowEma;
		private DataSeries      diffSeries;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Navy, 2), PlotStyle.Bar, "Diff"));
			Add(new Line(Color.DarkGray, 0, "Zero line"));

			fastEma	= new DataSeries(this);
			slowEma	= new DataSeries(this);
			diffSeries = new DataSeries(this);
		}

		/// <summary>
		/// Calculates the indicator value(s) at the current index.
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (CurrentBar < Slow) return;						
			
				fastEma.Set((2.0 / (1 + Fast)) * Input[0] + (1 - (2.0 / (1 + Fast))) * fastEma[1]);
				slowEma.Set((2.0 / (1 + Slow)) * Input[0] + (1 - (2.0 / (1 + Slow))) * slowEma[1]);
				diffSeries.Set(fastEma[0] - slowEma[0]); 
				
				double macd		= fastEma[0] - slowEma[0];
				double macdAvg	= SMA(diffSeries, Smooth)[0];
				
				Diff.Set(macd - macdAvg);
			
		}

		#region Properties
		
        /// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Diff
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for fast EMA")]
		[Category("Parameters")]
		public int Fast
		{
			get { return fast; }
			set { fast = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for slow EMA")]
		[Category("Parameters")]
		public int Slow
		{
			get { return slow; }
			set { slow = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for smoothing")]
		[Category("Parameters")]
		public int Smooth
		{
			get { return smooth; }
			set { smooth = Math.Max(1, value); }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private OsMA[] cacheOsMA = null;

        private static OsMA checkOsMA = new OsMA();

        /// <summary>
        /// OsMA
        /// </summary>
        /// <returns></returns>
        public OsMA OsMA(int fast, int slow, int smooth)
        {
            return OsMA(Input, fast, slow, smooth);
        }

        /// <summary>
        /// OsMA
        /// </summary>
        /// <returns></returns>
        public OsMA OsMA(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            checkOsMA.Fast = fast;
            fast = checkOsMA.Fast;
            checkOsMA.Slow = slow;
            slow = checkOsMA.Slow;
            checkOsMA.Smooth = smooth;
            smooth = checkOsMA.Smooth;

            if (cacheOsMA != null)
                for (int idx = 0; idx < cacheOsMA.Length; idx++)
                    if (cacheOsMA[idx].Fast == fast && cacheOsMA[idx].Slow == slow && cacheOsMA[idx].Smooth == smooth && cacheOsMA[idx].EqualsInput(input))
                        return cacheOsMA[idx];

            OsMA indicator = new OsMA();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Fast = fast;
            indicator.Slow = slow;
            indicator.Smooth = smooth;
            indicator.SetUp();

            OsMA[] tmp = new OsMA[cacheOsMA == null ? 1 : cacheOsMA.Length + 1];
            if (cacheOsMA != null)
                cacheOsMA.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheOsMA = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// OsMA
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.OsMA OsMA(int fast, int slow, int smooth)
        {
            return _indicator.OsMA(Input, fast, slow, smooth);
        }

        /// <summary>
        /// OsMA
        /// </summary>
        /// <returns></returns>
        public Indicator.OsMA OsMA(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            return _indicator.OsMA(input, fast, slow, smooth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// OsMA
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.OsMA OsMA(int fast, int slow, int smooth)
        {
            return _indicator.OsMA(Input, fast, slow, smooth);
        }

        /// <summary>
        /// OsMA
        /// </summary>
        /// <returns></returns>
        public Indicator.OsMA OsMA(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.OsMA(input, fast, slow, smooth);
        }

    }
}
#endregion
